"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const platform_1 = require("@rxstack/platform");
const exceptions_1 = require("@rxstack/exceptions");
const _ = require("lodash");
const core_1 = require("@rxstack/core");
const utils_1 = require("./utils");
exports.softDelete = (options) => {
    return (event) => __awaiter(void 0, void 0, void 0, function* () {
        utils_1.restrictToOperations(event.eventType, [platform_1.OperationEventsEnum.PRE_EXECUTE]);
        options = _.merge({ deleteField: 'deletedAt', addOnCreate: false }, options);
        const metadata = event.metadata;
        const service = event.injector.get(metadata.service);
        switch (metadata.type) {
            case platform_1.ResourceOperationTypesEnum.CREATE:
            case platform_1.ResourceOperationTypesEnum.BULK_CREATE:
                addDeleteField(event, options);
                break;
            case platform_1.ResourceOperationTypesEnum.LIST:
            case platform_1.ResourceOperationTypesEnum.PATCH:
            case platform_1.ResourceOperationTypesEnum.BULK_REMOVE:
                modifyCriteria(event, options);
                break;
            case platform_1.ResourceOperationTypesEnum.UPDATE:
            case platform_1.ResourceOperationTypesEnum.GET:
                validateObject(event, options);
                break;
            case platform_1.ResourceOperationTypesEnum.REMOVE:
                validateObject(event, options);
                yield handleRemove(event, options, service);
                break;
        }
    });
};
const addDeleteField = (event, options) => {
    if (options.addOnCreate) {
        const data = event.request.body;
        _.isArray(data) ? data.forEach((v) => _.set(v, options.deleteField, null))
            : _.set(data, options.deleteField, null);
    }
};
const modifyCriteria = (event, options) => {
    const q = { [options.deleteField]: { '$eq': null } };
    if (event.request.attributes.has('query')) {
        _.merge(event.request.attributes.get('query'), { where: q });
    }
    else if (event.request.attributes.has('criteria')) {
        _.merge(event.request.attributes.get('criteria'), q);
    }
    else {
        throw new exceptions_1.MethodNotAllowedException(`Query or Criteria are not set in ${event.metadata.name}`);
    }
};
const validateObject = (event, options) => {
    if (_.isDate(_.get(event.getData(), options.deleteField))) {
        throw new exceptions_1.NotFoundException();
    }
};
const handleRemove = (event, options, service) => __awaiter(void 0, void 0, void 0, function* () {
    const id = _.get(event.getData(), service.options.idField);
    const data = _.merge(event.getData(), { [options.deleteField]: new Date() });
    yield service.updateOne(id, data);
    event.response = new core_1.Response(null, 204);
});
//# sourceMappingURL=soft-delete.js.map